// DirFileSys.cpp : SP99 Source File
//
// Defines behaviours for : directory routines
// 
// COPYRIGHT  1989 - 1998, ICR Software, Inc.
// All rights reserved.
// Created  : 04/05/1997
// Modified : 09/03/1998 07:42:23 PM
//

#include "stdafx.h"

#define AFXAPI_DATA __based(__segname("_DATA"))

#include <io.h>
#include <shlobj.h>
#include "DirFileSys.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW

// Constructor
//
CFileSystem::CFileSystem()
{
	// Initialize instance variables
	m_nMaxFileNameLength = 1024;
} // CFileSystem


// Destructor
//
CFileSystem::~CFileSystem()
{
	// No code.
} // ~CFileSystem

BOOL CFileSystem::LoadAniCursor(UINT nType)
{
	//	char * szName;
	char szName[16];
	HCURSOR hCursor;
	HANDLE hResInfo, hRes;
	LPSTR lpRes;
	
	switch (nType)
	{
		case 0:
			hCursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
			break;
		case 1:
			// locate the ANI resource
			strcpy(szName, "IDR_WAIT");
			hResInfo = ::FindResource(NULL, szName, "ANI");
			// locate failed - return FALSE
			if (!hResInfo)
				return FALSE;
			
			// otherwise, load the ANI resource
			hRes = ::LoadResource(NULL, (HRSRC__ *)hResInfo);
			// load failed - return FALSE
			if (!hRes)
				return FALSE;
			
			// lock the resource while playing so windows can't dump it
			lpRes = (char *)::LockResource(hRes);
			if (lpRes)
			{
				hCursor = ::SetCursor(LoadCursor(AfxGetInstanceHandle(), szName));
				::UnlockResource(hRes);
				::FreeResource(hRes);
			}
			break;
	}
	return 0;
} // LoadAniCursor

/***************************************************************************

	Name:    GetFileStatus

	Purpose: To return the status information about the specified file.
			 See the MFC help on the CFileStatus struct for more information.

	Ret Val:  TRUE: If no error.
			  FALSE: If error(ie. file does not exist).

	Example:
			  CFileStatus Status;
			  BOOL bRetVal = fs.GetFileStatus("c:\\test.txt", Status);

	Notes:    None.

***************************************************************************/
BOOL CFileSystem::GetFileStatus(const CString& FileName, CFileStatus& FileStatus)
{
	return CFile::GetStatus(FileName, FileStatus);
} // GetFileStatus


void CFileSystem::VerifyPath(const CString& PathName)
{
	char szTemp[128];
	char *lpTemp;
	
	strcpy(szTemp, PathName);
	CStringList *pDirs = GetSubdirList(szTemp, FALSE);
	
	if (lpTemp = strrchr(szTemp, '\\')) 
	{    
		* (++lpTemp) = '\0';
		
		// for recursive auto batch only 
		if (_chdir(szTemp) == -1)
			VerifyPath(szTemp);
		
		_mkdir(PathName);
		_chdir(PathName);
	}              
}

void CFileSystem::RecurseCopyPath(const CString& InputPathName, const CString& OutputPathName, const BOOL bRecurseSubdirs)
{
	BOOL bRetVal;
	CString strSubDir, strTmpPath;
	char szTemp[128];
	char *lpTemp;
	
	// get the number of sub-directories
	CStringList *pDirs = GetSubdirList(InputPathName, FALSE);
	
	strcpy(szTemp, InputPathName);
	int nPos = InputPathName.ReverseFind('\\');
	if (nPos)
	{
		strSubDir = InputPathName.Mid(nPos);
		strTmpPath = OutputPathName + strSubDir;
		bRetVal = MakePath(strTmpPath);
		
		if (lpTemp = strrchr(szTemp, '\\')) 
		{    
			* (++lpTemp) = '\0';
			
			// for recursive auto batch only 
			if (_chdir(szTemp) == -1)
			{
				;
			}
		}
	}
#if 0
	if (lpTemp = strrchr(szTemp, '\\')) 
	{    
		* (++lpTemp) = '\0';
		
		// for recursive auto batch only 
		if (_chdir(szTemp) == -1)
			;
		VerifyPath(szTemp);
		
		_mkdir(PathName);
		_chdir(PathName);
	}
#endif
}
// 
//
CString CFileSystem::GetLastSubDirectory(const CString& PathName) 
{
	CString strSubDir;
	
	int nLen = PathName.GetLength();
	int nPos = PathName.ReverseFind('\\');
	if (nPos == nLen - 1)
	{
		// remove last \\ and retry
		strSubDir = PathName.Left(nPos);
		nPos = strSubDir.ReverseFind('\\');
		if (nPos)
		{
			strSubDir = PathName.Left(nPos + 1);
		}
	}
	else if (nPos)
	{
		strSubDir = PathName.Mid(nPos + 1);
	}
	
	return strSubDir;
}

CStringList *
CFileSystem::MakeSubdirList(const CString& SearchDir, const CString& OutputDir, const BOOL bPathInName)
{
	CFileStatus FileStatus;
	
	// Read the directory list
	CStringList *  pDirList = new CStringList();
	
	CString SearchPath = AppendWildcard(SearchDir, "*.*");
	CString *  pString = GetDirectoryEntry(SearchPath, directory);
	while (pString != NULL)
	{
		CString  String;
		CString  FullPath;
		CString  CurDir = GetPath(SearchPath);
		_stprintf(FullPath.GetBufferSetLength(1024), _T("%s%s"), (const TCHAR *)CurDir, (const TCHAR *)(*pString));
		FullPath.ReleaseBuffer(-1);
		if (bPathInName)
		{
			_stprintf(String.GetBufferSetLength(1024), _T("%s%s"), (const TCHAR *)CurDir, (const TCHAR *)(*pString));
			String.ReleaseBuffer(-1);
		} // if
		else
		{
			_stprintf(String.GetBufferSetLength(1024), _T("%s"), (const TCHAR *)(*pString));
			String.ReleaseBuffer(-1);
		} // else
		
		// If it's not one of the special directories.
		if (*pString != "."  &&  *pString != "..")
		{
			// Get the file type and make sure it's a directory before we add it to the list.
			CFile::GetStatus((const TCHAR *) FullPath, FileStatus);
			if (FileStatus.m_attribute & directory)
			{
				pDirList->AddTail((const TCHAR *) String);
			} // if
		} // if
		
		// Delete the string we got back from GetDirectoryEntry
		
		delete pString;
		pString = NULL;
		
		pString = GetDirectoryEntry("", directory);
	} // while
	
#ifdef WIN32
	if (m_hFind != INVALID_HANDLE_VALUE)
	{
		VERIFY(FindClose(m_hFind));
		m_hFind = INVALID_HANDLE_VALUE;
	} // if
#endif
	
	return pDirList;
} // GetSubdirList


//***************************************************************************
//
// Name:    GetFileCreateTime
//
// Purpose: To return the creation time of the specified file.
//
// Ret Val: TRUE: If no error.
//          FALSE: If error (ie. file does not exist).
//
// Example:
//          CTime CreateTime;
//          BOOL bRetVal = fs.GetFileCreateTime("c:\\test.txt", CreateTime);
//
// Notes:   None.
//
//***************************************************************************
BOOL CFileSystem::GetFileCreateTime(const CString& FileName, CTime& time)
{
	CFileStatus FileStatus;
	if (CFile::GetStatus(FileName, FileStatus))
	{
		time = FileStatus.m_ctime;
		return TRUE;
	} // if
	
	return FALSE;
} // GetFileCreateTime


//***************************************************************************
//
// Name:    GetFileModifyTime
//
// Purpose:  To return the last modification time of the specified file.
//
// Ret Val: TRUE: If no error.
//          FALSE: If error (ie. file does not exist).
//
// Example:
//          CTime ModifyTime;
//          BOOL bRetVal = fs.GetFileModifyTime("c:\\test.txt", ModifyTime);
//
// Notes:   None.
//
//***************************************************************************
BOOL CFileSystem::GetFileModifyTime(const CString& FileName, CTime& time)
{
	CFileStatus FileStatus;
	if (CFile::GetStatus(FileName, FileStatus))
	{
		time = FileStatus.m_mtime;
		return TRUE;
	} // if
	
	return FALSE;
} // GetFileModifyTime


//***************************************************************************
//
// Name:    GetFileAccessTime
//
// Purpose: To return the time of last access for the specified file.
//
// Ret Val: TRUE: If no error.
//          FALSE: If error (ie. file does not exist).
//
// Example:
//          CTime AccessTime;
//          BOOL bRetVal = fs.GetFileAccessTime("c:\\test.txt", AccessTime);
//
// Notes:   None.
//
//***************************************************************************
BOOL CFileSystem::GetFileAccessTime(const CString& FileName, CTime& time)
{
	CFileStatus FileStatus;
	if (CFile::GetStatus(FileName, FileStatus))
	{
		time = FileStatus.m_atime;
		return TRUE;
	} // if
	
	return FALSE;
} // GetFileAccessTime


//***************************************************************************
//
// Name:    GetFileSize
//
// Purpose: To return the size of the specified file in bytes.
//
// Ret Val: TRUE: If no error.
//          FALSE: If error (ie. file does not exist).
//
// Example:
//          unsigned long lSize = 0;
//          BOOL bRetVal = fs.GetFileSize("c:\\test.txt", lSize);
//
// Notes:   None.
//
//***************************************************************************
BOOL CFileSystem::GetFileSize(const CString& FileName, unsigned long& lSize)
{
	CFileStatus FileStatus;
	if (CFile::GetStatus(FileName, FileStatus))
	{
		lSize = (unsigned long)FileStatus.m_size;
		return TRUE;
	} // if
	
	return FALSE;
} // GetFileSize


//***************************************************************************
//
// Name:    GetFileAttribute
//
// Purpose: To return the attribute bits associated with the file.
//
// Ret Val: TRUE: If no error.
//          FALSE: If error (ie. file does not exist).
//
// Example:
//          BOOL bRetVal = fs.GetFileAttribute("c:\\test.txt", bAttr);
//
// Notes:   See the Attribute enum in filesys.h for specific attributes.
//
//***************************************************************************
BOOL CFileSystem::GetFileAttribute(const CString& FileName, BYTE& nAttribute)
{
	CFileStatus FileStatus;
	if (CFile::GetStatus(FileName, FileStatus))
	{
		nAttribute = FileStatus.m_attribute;
		return TRUE;
	} // if
	
	return FALSE;
} // GetFileAttribute


#ifdef WIN32
//***************************************************************************
//
// Name:    GetADirectoryEntry
//
// Purpose: Function used in Win32 to get get directory entries with the
//          correct attributes set.
//
// Ret Val: TRUE: If no error.
//          FALSE: Error occurred.
//
// Example:
//
// Notes:   None
//
//***************************************************************************
BOOL CFileSystem::GetADirectoryEntry(CString &fileName, const CString& Wildcard /* = "" */, const unsigned long eFileAttrib /* = normal */)
{
	BOOL      bRetVal = TRUE;
	
	// If they passed in a Wildcard, we want to lookup the first entry.
	if (Wildcard != "")
	{
		m_hFind = FindFirstFile((const TCHAR *)Wildcard, &m_FileInfo);
		if (m_hFind == INVALID_HANDLE_VALUE)
		{
			bRetVal = FALSE;
		} // if
	} // if
	else
	{
		bRetVal = FindNextFile(m_hFind, &m_FileInfo);
	} // else
	
	
	if (bRetVal == FALSE)
	{
		// Error occurred.
		fileName = "";
		return FALSE;
	} // if
	
	if (m_FileInfo.dwFileAttributes & eFileAttrib)
	{
		// Create a string and copy the name to it.
		memcpy(fileName.GetBufferSetLength(m_nMaxFileNameLength), m_FileInfo.cFileName, _tcsclen(m_FileInfo.cFileName) + 1);
		fileName.ReleaseBuffer(-1);
	} // if
	else
	{
		fileName = "";
	} // else
	
	return TRUE;
} // GetADirectoryEntry
#endif


//***************************************************************************
//
// Name:     GetDirectoryEntry
//
// Purpose:  To return the next directory entry based on a wildcard, etc...
//
// Example:  CString *pFileName = fs.GetDirectoryEntry("*.txt", hidden);
//
// Notes:    None.
//
//***************************************************************************
CString * CFileSystem::GetDirectoryEntry(const CString& Wildcard /* = "" */, const unsigned long eFileAttrib /* = normal */)
{
#ifdef WIN32
	
	// Turn off critical error handler.
	UINT nPrevErrorMode = ::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	BOOL      bRetVal = TRUE;
	
	CString fileName;
	CString l_Wildcard = Wildcard;
	while (bRetVal  &&  fileName == "")
	{
		bRetVal = GetADirectoryEntry(fileName, l_Wildcard, eFileAttrib);
		l_Wildcard = "";
	} // while
	
	// Turn on critical error handler.
	::SetErrorMode(nPrevErrorMode);
	
	if (bRetVal == FALSE  &&  fileName == "")
	{
		return NULL;
	} // if
	
	// Create a string and copy the name to it.
	CString *pString = new CString(fileName);
	return pString;
	
#else
	
	int nRetVal;
	
	// Turn off DOS critical error handler.
	::SetErrorMode(1);
	
	// If they passed in a Wildcard, we want to lookup the first entry.
	if (Wildcard != "")
	{
		nRetVal = _dos_findfirst((const char *) Wildcard, (int) eFileAttrib, &m_FileInfo);
	} // if
	else
	{
		nRetVal = _dos_findnext(&m_FileInfo);
	} // else
	
	if (nRetVal != 0)
	{
		// Turn on DOS critical error handler.
		::SetErrorMode(0);
		
		// Error occurred, return NULL.
		return NULL;
	} // if
	
	// Create a string and copy the name to it.
	CString *pString = new CString();
	memcpy(pString->GetBufferSetLength(m_nMaxFileNameLength), m_FileInfo.name, 13);
	pString->ReleaseBuffer(-1);
	
	// Make the filename lower case.
	pString->MakeLower();
	
	// Turn on DOS critical error handler.
	::SetErrorMode(0);
	
	return pString;
	
#endif
} // GetDirectoryEntry


//***************************************************************************
//
// Name:     GetCurrentDirectory
//
// Purpose:  To get the current working directory.
//
// Example:  CString CurrentDir = fs.GetCurrentDirecory();
//
// Notes:    None
//
//***************************************************************************
CString CFileSystem::GetCurrentDirectory(const CString& FileSystem /* = "" */)
{
	// Turn off DOS critical error handler.
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	CString CurrentFileSystem = GetCurrentFileSystem();
	if (FileSystem != "")
	{
		if (!ChangeFileSystem(FileSystem))
		{
			return "";
		} // if
	} // if
	
	CString String;
	
#ifdef WIN32
	
	DWORD dwRetVal = ::GetCurrentDirectory(m_nMaxFileNameLength, String.GetBufferSetLength(m_nMaxFileNameLength));
	String.ReleaseBuffer(-1);
	
	if (dwRetVal == 0)
	{
		String = "";
	} // if
	
#else
	char *pRetVal = _tgetcwd(String.GetBufferSetLength(m_nMaxFileNameLength), m_nMaxFileNameLength);
	String.ReleaseBuffer(-1);
	
	// Make the string lower case.
	String.MakeLower();
	
	// If an error occured, clean up and return NULL.
	if (pRetVal == 0)
	{
		String = "";
	} // if
#endif
	
	if (FileSystem != "")
	{
		VERIFY(ChangeFileSystem(CurrentFileSystem));
	} // if
	
	// Turn on DOS critical error handler.
	::SetErrorMode(0);
	
	return String;
} // GetCurrentDirectory


//***************************************************************************
//
// Name:    ChangeDirectory
//
// Purpose: To change the current working directory.
//
// Example: BOOL bRetVal = fs.ChangeDirectory("c:\\foo\\bar");
//
// Notes:   None.
//
//***************************************************************************
BOOL CFileSystem::ChangeDirectory(const CString& NewDirectory)
{
	CString DirName = NewDirectory;
	
	// Remove any trailing slashes.
	int nLength = DirName.GetLength();
	if (nLength  &&
		DirName.GetAt(nLength - 1) == '\\')
	{
		DirName = DirName.Left(nLength - 1);
	} // if
	
#ifdef WIN32
	return SetCurrentDirectory((const TCHAR *) DirName);
#else
	int nRetVal = _tchdir((const TCHAR *) DirName);
	if (nRetVal == -1)
	{
		return FALSE;
	} // if
	
	return TRUE;
#endif
} // ChangeDirectory


//***************************************************************************
//
// Name:    RenameDirectory
//
// Purpose: To rename a directory.
//
// Ret Val: TRUE : No error, or OldName == NewName.
//          FALSE : Error.
//
// Example: BOOL bRetVal = fs.RenameDirectory("c:\\foo", "c:\\bar");
//
// Notes:   Using WIN16, if the new directory name is longer than 8.3,
//          the parts that are too long will be truncated.
//
//***************************************************************************
BOOL CFileSystem::RenameDirectory(const CString& OldName, const CString& NewName)
{
	if (OldName == NewName)
	{
		return TRUE;
	} // if
	
	if (!DirectoryExists(OldName)  ||  DirectoryExists(NewName))
	{
		return FALSE;
	} // if
	
	TRY
	{
		CFile::Rename((const TCHAR *) OldName, (const TCHAR *) NewName);
	} // TRY
	CATCH(CFileException, Exception)
	{
		return FALSE;
	} // CATCH
	END_CATCH
		
		return TRUE;
} // RenameDirectory


//***************************************************************************
//
// Name:    MakeDirectory
//
// Purpose: To make a directory.
//
// Example: BOOL bRetVal = fs.MakeDirectory("c:\\foo\\bar");
//
// Notes:   None
//
//***************************************************************************
BOOL CFileSystem::MakeDirectory(const CString& NewDirectory)
{
#ifdef WIN32
	
	SECURITY_ATTRIBUTES security_attrib;
	security_attrib.nLength = sizeof(SECURITY_ATTRIBUTES);
	security_attrib.lpSecurityDescriptor = NULL;
	security_attrib.bInheritHandle = TRUE;
	
	BOOL bRetVal = CreateDirectory((const TCHAR *) NewDirectory, &security_attrib);
	return bRetVal;
	
#else
	
	int nRetVal = _tmkdir((const char *) NewDirectory);
	if (nRetVal == -1)
	{
		return FALSE;
	} // if
	
	return TRUE;
	
#endif
} // MakeDirectory


//***************************************************************************
//
// Name:    MakePath
//
// Purpose: To make all the directories in a given path.  If any of the
//          directories exist, the creation continues with lower level
//          directories.
//
// Ret Val: TRUE : No error.
//          FALSE : Unable to create path.
//                  Path already exists.
//                  No path given.
//
// Example: BOOL bRetVal = fs.MakePath("c:\\foo\\bar\\what");
//
// Notes:    None
//
//***************************************************************************
BOOL CFileSystem::MakePath(const CString& NewDirectory)
{
	CString NewDir = NewDirectory;  // Copy string for manipulation
	CString  DirName;
	BOOL    bRetVal = TRUE;
	
	// Error if no directory specified.
	if (NewDir.GetLength() == 0)
	{
		return FALSE;
	} // if
	
	// Make sure the directory name ends in a slash
	if (NewDir[NewDir.GetLength() - 1] != '\\')
	{
		NewDir = NewDir + '\\';
	} // if
	
	// Create each directory in the path
	UINT  nIndex = 0;
	BOOL  bDone = FALSE;
	while (!bDone)
	{
		// Extract one directory
		nIndex = NewDir.Find('\\');
		if (nIndex != -1)
		{
			DirName = DirName + NewDir.Left(nIndex);
			NewDir = NewDir.Right(NewDir.GetLength() - nIndex - 1);
			
			// The first time through, we might have a drive name
			if (DirName.GetLength() >= 1  &&  DirName[DirName.GetLength() - 1] != ':')
			{
				bRetVal = MakeDirectory(DirName);
			} // if
			DirName = DirName + '\\';
		} // if
		else
		{
			// We're finished
			bDone = TRUE;
		} // else
	} // while
	
	// Return the last MakeDirectory() return value.
	return bRetVal;
} // MakePath


//***************************************************************************
//
// Name:    DeleteDirectory
//
// Purpose: To delete a directory.  Optionally, all lower level files
//          and directories can be deleted.
//
// Ret Val: TRUE : Successfully deleted directory.
//          FALSE : Failed to delete directory.
//
// Example: BOOL bRetVal = fs.DeleteDirectory("c:\\foo\\bar", TRUE);
//
// Notes:   If bDeleteFilesAndDirs == FALSE, directory must be empty or
//          an error will occur and FALSE will be returned.
//
//***************************************************************************
BOOL CFileSystem::DeleteDirectory(const CString& Directory, const BOOL bDeleteFilesAndDirs /* = FALSE */)
{
	if (bDeleteFilesAndDirs)
	{
		CStringList *pSubDirs = GetSubdirList(Directory);
		for (POSITION pos = pSubDirs->GetHeadPosition(); pos != NULL; )
		{
			CString DirName = pSubDirs->GetNext(pos);
			DeleteDirectory(DirName, bDeleteFilesAndDirs);
		} // for
		
		delete pSubDirs;
		pSubDirs = NULL;
		
		DeleteFiles(AppendWildcard(Directory, "*.*"), allfiles);
	} // if
	
#ifdef WIN32
	
	BOOL bRetVal = RemoveDirectory((const TCHAR *) Directory);
	return bRetVal;
	
#else
	
	int nRetVal = _trmdir((const char *) Directory);
	if (nRetVal == -1)
	{
		return FALSE;
	} // if
	
	return TRUE;
#endif
} // DeleteDirectory


//***************************************************************************
//
// Name:    GetDirectorySize
//
// Purpose: To return the size (in bytes) of the files in the specified
//          directory.
//
// Example: LONG lSize = fs.GetDirectorySize("c:\\foo\\bar", "*.*", TRUE);
//
// Notes:   If an error occurs in reading the status of a file, that
//          file is skipped and not counted as part of the size.
//          There is currently not way to find out if an error of this
//          kind occured.
//
//***************************************************************************
LONG CFileSystem::GetDirectorySize(const CString& Directory /* = "" */, const CString& WildCard /* = "*.*" */, const BOOL bRecurseSubdirs /* = FALSE */)
{
	unsigned long lSize = 0;
	
	// Do all the subdirectories first...
	if (bRecurseSubdirs)
	{
		CStringList *pSubDirs = GetSubdirList(Directory);
		for (POSITION pos = pSubDirs->GetHeadPosition(); pos != NULL; )
		{
			CString DirName = pSubDirs->GetNext(pos);
			lSize += GetDirectorySize(DirName, WildCard, bRecurseSubdirs);
		} // for
		
		delete pSubDirs;
		pSubDirs = NULL;
	} // if
	
	// Find the sizes of all the files in the specified directory.
	CStringList *pFileList = GetFileList(AppendWildcard(Directory, WildCard), allfiles);
	for (POSITION pos = pFileList->GetHeadPosition(); pos != NULL; )
	{
		CFileStatus status;
		CString FileName = pFileList->GetNext(pos);
		if (CFile::GetStatus(FileName, status))
		{
			lSize += (unsigned long) status.m_size;
		} // if
	} // for
	
	delete pFileList;
	pFileList = NULL;
	
	return lSize;
} // GetDirectorySize


//***************************************************************************
//
// Name:    GetCurrentFileSystem
//
// Purpose: To return a string containing the current file system name.
//
// Example: CString FileSystem = fs.GetCurrentFileSystem();
//
// Notes:   None
//
//***************************************************************************
CString CFileSystem::GetCurrentFileSystem()
{
	unsigned int nDrive = 0;
	char cDrive = 'a';
	
#ifdef WIN32
	nDrive = _getdrive();
#else
	_dos_getdrive(&nDrive);
#endif
	cDrive = (char)('a' + nDrive - 1);
	CString String = cDrive;
	String += ":\\";
	
	return String;
} // GetCurrentFileSystem


//***************************************************************************
//
// Name:    ChangeFileSystem
//
// Purpose: To change the current file system.
//
// Example: BOOL bRetVal = fs.ChangeFileSystem('c');
//
// Notes:   Obsolete.  The version that takes a CString parameter
//          should be used instead.  This change was made in preparation
//          for the addition of Windows NT support.
//
//***************************************************************************
BOOL CFileSystem::ChangeFileSystem(const char cFileSystem)
{
	unsigned int nNewDrive;
	BOOL bRetVal = FALSE;
	
	// Turn off DOS critical error handler.
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	nNewDrive = _totupper(cFileSystem) - 'A' + 1;
	
	if (nNewDrive >= 0  &&  nNewDrive <= 26)
	{
#ifdef WIN32
		
		if (_chdrive(nNewDrive) == 0)
		{
			bRetVal = TRUE;
		} // if
		
#else
		// Attempt change filesystems.
		unsigned int nNumDrives;		// ignored return value
		_dos_setdrive(nNewDrive, &nNumDrives);
		
		// Make sure the change actually happened (it's the only way).
		unsigned int nCurrentDrive;
		_dos_getdrive(&nCurrentDrive);
		if (nCurrentDrive == nNewDrive)
		{
			bRetVal = TRUE;
		} // if
		
#endif
	} // if
	
	// Turn on DOS critical error handler.
	::SetErrorMode(0);
	
	return bRetVal;
} // ChangeFileSystem


//***************************************************************************
//
// Name:    ChangeFileSystem
//
// Purpose: To change the current file system.
//
// Example: BOOL bRetVal = fs.ChangeFileSystem("c:\\");
//
// Notes:   Expects a filesystem name of the form "X:\".
//
//***************************************************************************
BOOL CFileSystem::ChangeFileSystem(const CString& FileSystem)
{
#ifdef WIN32
	
	return SetCurrentDirectory((const TCHAR *) FileSystem);
	
#else
	
	BOOL bRetVal = FALSE;
	
	if (FileSystem.GetLength() > 0)
	{
		char cFileSystem = FileSystem[0];
		bRetVal = ChangeFileSystem(cFileSystem);
	} // if
	
	return bRetVal;
	
#endif
} // ChangeFileSystem


//***************************************************************************
//
// Name:    GetFileSystemList
//
// Purpose: To return a list of available file systems (ie. drives).
//
// Example: CStringList *pFSList = fs.GetFileSystemList();
//
// Notes:   The user must free the returned list.
//
//***************************************************************************
CStringList  *
CFileSystem::GetFileSystemList()
{
	CStringList *pStringList = new CStringList();
	
	// Turn off DOS critical error handler.
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	for (int nDriveNum = 0; nDriveNum < 26; ++nDriveNum)
	{
#ifdef WIN32
		
		CString DriveName = (char)('a' + nDriveNum);
		DriveName = DriveName + ":\\";
		if (GetDriveType((const TCHAR *)DriveName) > 1)
		{
			CString DriveName = (char)('a' + nDriveNum);
			DriveName = DriveName + ":\\";
			pStringList->AddTail((const TCHAR *)DriveName);
		} // if
		
#else
		
		if (GetDriveType(nDriveNum) != 0)
		{
			CString DriveName = (char)('a' + nDriveNum);
			DriveName = DriveName + ":\\";
			pStringList->AddTail((const char *)DriveName);
		} // if
		
#endif
	} // for
	
	// Turn on DOS critical error handler.
	::SetErrorMode(0);
	
	return pStringList;
} // GetFileSystemList


//***************************************************************************
//
// Name:    GetVolumeLabel
//
// Purpose: To get the volume label for a filesystem.
//
// Example: CString VolumeLabel = fs.GetVolumneLabel("c:\\");
//
// Notes:   None
//
//***************************************************************************
CString CFileSystem::GetVolumeLabel(const CString& FileSystem)
{
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
#ifdef WIN32
	
	CString  VolumeLabel;
	DWORD    dwVolumeSerialNumber;
	DWORD    dwMaximumComponentLength;
	DWORD    dwFileSystemFlags;
	TCHAR    FileSystemName[255];
	
	if (!GetVolumeInformation((const TCHAR *) FileSystem,
		VolumeLabel.GetBufferSetLength(255), 255,
		&dwVolumeSerialNumber,
		&dwMaximumComponentLength,
		&dwFileSystemFlags,
		FileSystemName, 255))
	{
		VolumeLabel = _T("");
	} // if
	
#else
	
	CString VolumeLabel = "";
	CString *pVolLabel = GetDirectoryEntry(AppendWildcard(FileSystem, "*.*"), volume);
	if (pVolLabel != NULL)
	{
		VolumeLabel = *pVolLabel;
		
		delete pVolLabel;
		pVolLabel = NULL;
	} // if
	
#endif
	
	::SetErrorMode(0);
	
	return VolumeLabel;
} // GetVolumeLabel


//***************************************************************************
//
// Name:    GetFileSystemType
//
// Purpose: To get the type (removable, network, etc...) of the specified
//          filesystem.
//
// Ret Val: 0 = Error.
//          DRIVE_REMOVABLE : Removable Disk (ie. floppy)
//          DRIVE_FIXED : Non-Removable Disk (ie. hard drive)
//          DRIVE_REMOTE : Network Disk (ie. NFS mounted, Netware volume)
//
// Example: LONG lType = fs.GetFileSystemType("c:\\");
//          ASSERT(lType == DRIVE_FIXED);
//
// Notes:   None
//
//***************************************************************************
LONG
CFileSystem::GetFileSystemType(const CString& FileSystem)
{
	LONG lType = DRIVE_UNDETERMINED;
	
#ifdef WIN32
	
	lType = GetDriveType((const TCHAR *) FileSystem);
	
#else
	
	if (FileSystem.GetLength() > 0)
	{
		char cFileSystem = (char) _totupper(FileSystem[0]);
		
		int nDriveNum = _totupper(cFileSystem) - 'A';
		
		if (nDriveNum >= 0  &&  nDriveNum <= 26)
		{
			::SetErrorMode(1);
			lType = GetDriveType(nDriveNum);
			::SetErrorMode(0);
		} // if
	} // if
	
#endif
	
	return lType;
} // GetFileSystemType


//***************************************************************************
//
// Name:  GetFreeSpace
//
//***************************************************************************
LONG
CFileSystem::GetFreeSpace(const CString& FileSystem)
{
#ifdef WIN32
	
	LONG  lRetVal;
	DWORD dwSectorsPerCluster;
	DWORD dwBytesPerSector;
	DWORD dwFreeClusters;
	DWORD dwClusters;
	
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	if (GetDiskFreeSpace((const TCHAR *) FileSystem,
		&dwSectorsPerCluster,
		&dwBytesPerSector,
		&dwFreeClusters,
		&dwClusters))
	{
		lRetVal = dwSectorsPerCluster * dwBytesPerSector * dwFreeClusters;
	} // if
	else
	{
		lRetVal = -1;
	} // else
	
	::SetErrorMode(0);
	
	return lRetVal;
	
#else
	
	unsigned long lFreeSpace = -1;
	struct _diskfree_t diskspace;
	
	if (FileSystem.GetLength() > 0)
	{
		char cFileSystem = (char) _totupper(FileSystem[0]);
		
		int nDriveNum = _totupper(cFileSystem) - 'A';
		
		if (nDriveNum >= 0  &&  nDriveNum <= 26)
		{
			::SetErrorMode(1);
			if (_dos_getdiskfree(nDriveNum + 1,	&diskspace) == 0)
			{
				lFreeSpace = (unsigned long) diskspace.avail_clusters *
					(unsigned long) diskspace.sectors_per_cluster	*
					(unsigned long) diskspace.bytes_per_sector;
			} // if
			::SetErrorMode(0);
		} // if
	} // if
	
	return lFreeSpace;
	
#endif
} // GetFreeSpace


//***************************************************************************
//
// Name:  GetTotalSpace
//
//***************************************************************************
LONG
CFileSystem::GetTotalSpace(const CString& FileSystem)
{
#ifdef WIN32
	
	LONG  lRetVal;
	DWORD dwSectorsPerCluster;
	DWORD dwBytesPerSector;
	DWORD dwFreeClusters;
	DWORD dwClusters;
	
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	if (GetDiskFreeSpace((const TCHAR *) FileSystem,
		&dwSectorsPerCluster,
		&dwBytesPerSector,
		&dwFreeClusters,
		&dwClusters))
	{
		lRetVal = dwSectorsPerCluster * dwBytesPerSector * dwClusters;
	} // if
	else
	{
		lRetVal = -1L;
	} // else
	
	::SetErrorMode(0);
	
	return lRetVal;
	
#else
	
	unsigned long lTotalSpace = -1;
	struct _diskfree_t diskspace;
	
	if (FileSystem.GetLength() > 0)
	{
		char cFileSystem = (char) _totupper(FileSystem[0]);
		
		int nDriveNum = _totupper(cFileSystem) - 'A';
		
		if (nDriveNum >= 0  &&  nDriveNum <= 26)
		{
			::SetErrorMode(1);
			if (_dos_getdiskfree(nDriveNum + 1,	&diskspace) == 0)
			{
				lTotalSpace = (unsigned long) diskspace.total_clusters *
					(unsigned long) diskspace.sectors_per_cluster	*
					(unsigned long) diskspace.bytes_per_sector;
			} // if
			::SetErrorMode(0);
		} // if
	} // if
	
	return lTotalSpace;
	
#endif
} // GetTotalSpace


//***************************************************************************
//
// Name:  IsReadOnlyFileSystem
//
//***************************************************************************
BOOL
CFileSystem::IsReadOnlyFileSystem(const CString& FileSystem)
{
	BOOL            bRetVal = TRUE;
	CFile           TestFile;
	CFileException  fileException;
	CString         TempFileName;
	
	// Turn off DOS critical error handler.
	::SetErrorMode(SEM_FAILCRITICALERRORS);
	
	// Get the temp file name
	TempFileName = "xxxxtest.999";
	
	// Try to create the temporary file.
	if (TestFile.Open(AppendWildcard(FileSystem, TempFileName), CFile::modeCreate, &fileException))
	{
		TestFile.Close();
		if (DeleteFile(AppendWildcard(FileSystem, TempFileName)))
		{
			bRetVal = FALSE;
		} // else
	} // if
	
	// Turn on DOS critical error handler.
	::SetErrorMode(0);
	
	return bRetVal;
} // IsReadOnlyFileSystem


//***************************************************************************
//
// Name:    RenameFile
//
// Purpose: To rename a file.
//
// Ret Val: TRUE : No error, or OldFileName == NewFileName.
//          FALSE : Error.
//
// Example: BOOL bRetVal = fs.RenameFile("c:\\foo.txt", "c:\\bar.doc");
//          BOOL bRetVal = fs.RenameFile("c:\\foo.txt", "c:\\what\\bar.doc");
//
// Notes:   A file can also be moved using this function by specifying a
//          different directory in the NewFileName.
//
//***************************************************************************
BOOL
CFileSystem::RenameFile(const CString& OldFileName, const CString& NewFileName)
{
	if (OldFileName == NewFileName)
	{
		return TRUE;
	} // if
	
	if (!FileExists(OldFileName)  ||  FileExists(NewFileName))
	{
		return FALSE;
	} // if
	
	TRY
	{
		CFile::Rename((const TCHAR *) OldFileName, (const TCHAR *) NewFileName);
	} // TRY
	CATCH(CFileException, Exception)
	{
		return FALSE;
	} // CATCH
	END_CATCH
		
		return TRUE;
} // RenameFile


//***************************************************************************
//
// Name:    DeleteFiles
//
// Purpose: To delete a set of files based on a wildcard file specification.
//
// Example: BOOL bRetVal = fs.DeleteFiles("c:\\foo\\bar\\*.txt");
//
//***************************************************************************
BOOL
CFileSystem::DeleteFiles(const CString& FileSpec, const unsigned long eFileAttrib /* = normal */)
{
	BOOL bRetVal = TRUE;
	CStringList *pDir = GetDirectory(FileSpec, eFileAttrib);
	
	for (POSITION pos = pDir->GetHeadPosition(); pos != NULL; )
	{
		CString FileName = pDir->GetNext(pos);
		if (DeleteFile(FileName) == FALSE)
		{
			bRetVal = FALSE;
		} // if
	} // for
	
	// Clean up.
	delete pDir;
	pDir = NULL;
	
	return bRetVal;
} // DeleteFiles


//***************************************************************************
//
// Name:    DeleteFile
//
// Purpose: To delete a file.
//
// Example:
//          BOOL bRetVal = fs.DeleteFile("c:\foo.txt");
//
// Notes:   None
//
//***************************************************************************
BOOL
CFileSystem::DeleteFile(const CString& FileName)
{
	TRY
	{
		CFile::Remove((const TCHAR *) FileName);
	} // TRY
	CATCH(CFileException, Exception)
	{
		return FALSE;
	} // CATCH
	END_CATCH
		
		return TRUE;
} // DeleteFile


//***************************************************************************
//
// Name:    CloseFile
//
// Purpose: To close a file.
//
// Ret Val: TRUE : File was closed successfully.
//          FALSE : Error occured.
//
// Notes:   None
//
//***************************************************************************
BOOL
CFileSystem::CloseFile(CFile *pFile) const
{
	BOOL bRetVal = TRUE;
	
	TRY
	{
		pFile->Close();
	} // TRY
	CATCH(CFileException, e)
	{
		bRetVal = FALSE;
	} // CATCH
	END_CATCH
		
		return bRetVal;
} // CloseFile


//***************************************************************************
//
// Name:    CopyFile
//
// Purpose: To copy a file.
//
// Ret Val: TRUE : File was copied successfully.
//          FALSE : Error occured.
//
// Example: BOOL bRetVal = fs.CopyFile("foo.txt", "a:\\bar.txt", 20480);
//
// Notes:   If the destination file exists, an error occurs.
//
//***************************************************************************
BOOL
CFileSystem::CopyFile(const CString& SourceFileName, const CString& DestFileName, const unsigned long lBuffSize /* = 10240 */)
{
	BOOL    bRetVal  = TRUE;
	char *  pBuff    = new char[lBuffSize];
	CFile Source;
	CFile Dest;
	
	// Check for exitance of the destination file.
	CFileStatus destStatus;
	if ((CFile::GetStatus(DestFileName, destStatus)))
	{
		delete[] pBuff;
		pBuff = NULL;
		
		return FALSE;
	} // if
	
	// Open the files, creating the destination.
	if (Source.Open((const TCHAR *) SourceFileName, CFile::modeRead))
	{
		if (Dest.Open((const TCHAR *) DestFileName, CFile::modeCreate | CFile::modeWrite))
		{
			ULONGLONG  dwLength = Source.GetLength();
			
			// Copy the data in the file.
			while (dwLength > 0)
			{
				UINT nRead = Source.Read(pBuff, (UINT) lBuffSize);
				if (nRead)
				{
					Dest.Write(pBuff, nRead);
				} // if
				
				dwLength -= nRead;
			} // while
			
			CloseFile(&Source);
			CloseFile(&Dest);
		} // if
		else
		{
			CloseFile(&Source);
			
			bRetVal = FALSE;
		} // else
	} // if
	else
	{
		bRetVal = FALSE;
	} // else
	
	delete[] pBuff;
	pBuff = NULL;
	
	return bRetVal;
} // CopyFile


//***************************************************************************
//
// Name:    CopyFiles
//
// Purpose: To copy a set of files based on a wildcard file specification.
//
// Example: BOOL bRetVal = fs.CopyFiles("c:\\foo\\bar\\*.txt", "c:\\foo2");
//
// Notes:   If an error occurs, the copy is aborted.  There is no way to
//          which if any of the files were copied.
//
//          Even if you are ignoring errors, the return value indicates
//          whether an error occured.
//
//***************************************************************************
BOOL CFileSystem::CopyFiles(const CString& FileSpec, const CString& DestPath, const BOOL bIgnoreErrors /* = FALSE */, const unsigned long eFileAttrib /* = normal */)
{
	BOOL bRetVal = TRUE;
	CStringList *pDir = GetDirectory(FileSpec, eFileAttrib);
	
	for (POSITION pos = pDir->GetHeadPosition(); pos != NULL; )
	{
		CString FileName = pDir->GetNext(pos);
		CString DestFileName = AppendWildcard(DestPath, GetFileName(FileName));
		
		if (CopyFile(FileName, DestFileName) == FALSE)
		{
			bRetVal = FALSE;
			
			if (bIgnoreErrors == FALSE)
			{
				break;
			} // if
		} // if
	} // for
	
	// Clean up.
	delete pDir;
	pDir = NULL;
	
	return bRetVal;
} // CopyFiles


//***************************************************************************
//
// Name:    CompareFiles
//
// Purpose: To compare the contents of two files to see if they are the
//          same.
//
// Ret Val: TRUE, if the files are the same.
//          FALSE, if the files are different, or an error occurs.
//
// Example: BOOL bRetVal = fs.CompareFiles("foo.txt", "bar.txt", 20480);
//
// Notes:   None
//
//***************************************************************************
BOOL
CFileSystem::CompareFiles(const CString& FileName1, const CString& FileName2, const unsigned long lBuffSize /* = 10240 */)
{
	BOOL    bRetVal  = TRUE;
	char *  pBuff1  = new char[lBuffSize];
	char *  pBuff2  = new char[lBuffSize];
	CFile   File1;
	CFile   File2;
	
	// Make sure we allocated the buffers
	if (!pBuff1  ||  !pBuff2)
	{
		if (pBuff1)
		{
			delete[] pBuff1;
		} // if
		
		if (pBuff2)
		{
			delete[] pBuff2;
		} // if
		
		return FALSE;
	} // if
	
	// Open the files.
	if (File1.Open((const TCHAR *) FileName1, CFile::modeRead | CFile::shareDenyWrite))
	{
		if (File2.Open((const TCHAR *) FileName2, CFile::modeRead | CFile::shareDenyWrite))
		{
			ULONGLONG  dwLength1 = File1.GetLength();
			ULONGLONG  dwLength2 = File2.GetLength();
			if (dwLength1 != dwLength2)
			{
				bRetVal = FALSE;
			} // if
			
			// Read the data in the file.
			while (bRetVal == TRUE  &&  dwLength1 > 0)
			{
				UINT nRead1 = File1.Read(pBuff1, (UINT) lBuffSize);
				UINT nRead2 = File2.Read(pBuff2, (UINT) lBuffSize);
				
				if (nRead1 != nRead2)
				{
					bRetVal = FALSE;
					break;              // break out of the while loop
				} // if
				
				if (memcmp(pBuff1, pBuff2, nRead1) != 0)
				{
					bRetVal = FALSE;
				} // if
				
				dwLength1 -= nRead2;
			} // while
			
			CloseFile(&File1);
			CloseFile(&File2);
		} // if
		else
		{
			CloseFile(&File1);
			
			bRetVal = FALSE;
		} // else
	} // if
	else
	{
		bRetVal = FALSE;
	} // else
	
	delete[] pBuff1;
	delete[] pBuff2;
	
	return bRetVal;
} // CompareFiles


//***************************************************************************
//
// Name:    CompareDirectories
//
//***************************************************************************
BOOL
CFileSystem::CompareDirectories(const CString& PathName1, const CString& PathName2, const BOOL bRecurseSubdirs /* = FALSE */, const BOOL bCompareFileContents /* = FALSE */, const unsigned long eFileAttrib /* = normal */)
{
	BOOL bRetVal = TRUE;
	
	
	// See if the two pathnames are the same.
	
	if (PathName1 == PathName2)
	{
		return TRUE;
	} // if
	
	
	// Get the first directory listing.
	
	CString CurrentDirectory = GetCurrentDirectory();
	if (ChangeDirectory(GetPath(PathName1)) == 0)
	{
		return FALSE;
	} // if
	CStringList *pDir1 = GetDirectory(GetFileName(PathName1), eFileAttrib, bRecurseSubdirs);
	if (ChangeDirectory(CurrentDirectory) == 0)
	{
		delete pDir1;
		pDir1 = NULL;
		return FALSE;
	} // if
	
	
	// Get the second directory listing.
	
	if (ChangeDirectory(GetPath(PathName2)) == 0)
	{
		delete pDir1;
		pDir1 = NULL;
		return FALSE;
	} // if
	CStringList *pDir2 = GetDirectory(GetFileName(PathName2), eFileAttrib, bRecurseSubdirs);
	if (ChangeDirectory(CurrentDirectory) == 0)
	{
		delete pDir1;
		pDir1 = NULL;
		
		delete pDir2;
		pDir2 = NULL;
		
		return FALSE;
	} // if
	
	
	// Compare the directory listings.
	
	if (pDir1->GetCount() != pDir2->GetCount())
	{
		bRetVal = FALSE;
	} // if
	else
	{
		Sort(pDir1);
		Sort(pDir2);
		
		POSITION pos2 = pDir2->GetHeadPosition();
		for (POSITION pos1 = pDir1->GetHeadPosition(); pos1 != NULL; )
		{
			CString String1 = pDir1->GetNext(pos1);
			CString String2 = pDir2->GetNext(pos2);
			TRACE(_T("CFileSystem::CompareDirectories %s %s\n"), (const TCHAR *) String1, (const TCHAR *)String2);
			
			if (String1 != String2)
			{
				bRetVal = FALSE;
				break;
			} // if
			else
			{
				if (bCompareFileContents)
				{
					if (!CompareFiles(AppendWildcard(GetPath(PathName1), String1), AppendWildcard(GetPath(PathName2), String2)))
					{
						bRetVal = FALSE;
						break;
					} // if
				} // if
			} // else
		} // for
	} // else
	
	
	// Clean up.
	delete pDir1;
	pDir1 = NULL;
	
	delete pDir2;
	pDir2 = NULL;
	
	return bRetVal;
} // CompareDirectories


//***************************************************************************
//
// Name:    GetFullPathName
//
// Example:  fs.GetFullPathName("c:\\foo\\bar\\what.txt");
//           fs.GetFullPathName("c:\\foo\\..\\bar\\what.txt");
//           fs.GetFullPathName("\\bar\\..\\what.txt");
//           fs.GetFullPathName("..\\bar\\.\\what.txt");
//           fs.GetFullPathName("c:..\\foo\\bar\\what.txt");
//
//***************************************************************************
CString
CFileSystem::GetFullPathName(const CString& PathAndFileName)
{
#ifdef WIN32
	
	LPTSTR pTemp;
	CString FullPathName;
	
	::GetFullPathName((const TCHAR *)PathAndFileName,
		m_nMaxFileNameLength,
		FullPathName.GetBufferSetLength(m_nMaxFileNameLength),
		&pTemp);
	
	FullPathName.ReleaseBuffer(-1);
	
	return FullPathName;
	
#else
	
	CString FullPathName;
	
	// Determine the drive.
	CString FileSystem = GetFileSystem(PathAndFileName);
	if (FileSystem == "")
	{
		FileSystem = GetCurrentFileSystem();
	} // if
	
	// Determine if the path is relative to the root of the filesystem
	// or relative to the current directory of the filesystem.
	int nIndex = PathAndFileName.Find(":\\");
	if (nIndex != -1)
	{
		FullPathName = FileSystem;
	} // if
	else if (PathAndFileName.GetLength()  &&  PathAndFileName[0] == '\\')
	{
		FullPathName = FileSystem;
	} // else if
	else
	{
		FullPathName = GetCurrentDirectory(FileSystem) + "\\";
	} // else
	
	CString PartialPath = GetPath(PathAndFileName + "\\", FALSE);
	if (PartialPath[0] == '\\')
	{
		PartialPath = PartialPath.Right(PartialPath.GetLength() - 1);
	} // if
	if (PartialPath.GetLength())
	{
		PartialPath = PartialPath.Left(PartialPath.GetLength() - 1);
	} // if
	
	FullPathName = FullPathName + PartialPath;
	
	
	// Change all the slash-dot-slash sequences to slashes.
	
	BOOL bDone = FALSE;
	while (!bDone)
	{
		int nIndex = FullPathName.Find("\\.\\");
		if (nIndex == -1)
		{
			bDone = TRUE;
		} // if
		else
		{
			FullPathName = FullPathName.Left(nIndex) + FullPathName.Right(FullPathName.GetLength() - nIndex - 2);
		} // else
	} // while
	
	
	// Handle all the slash-dot-dot-slash sequences.
	
	bDone = FALSE;
	while (!bDone)
	{
		int nIndex = FullPathName.Find("\\..\\");
		if (nIndex == -1)
		{
			bDone = TRUE;
		} // if
		else
		{
#if 0
			FullPathName = "";
#else
			PartialPath = FullPathName.Left(nIndex);
			int nIndex2 = PartialPath.ReverseFind('\\');
			if (nIndex2 == -1)
			{
				FullPathName = "";
			} // if
			else
			{
				FullPathName = PartialPath.Left(nIndex2) + FullPathName.Right(FullPathName.GetLength() - nIndex - 3);
			} // else
#endif
		} // else
	} // while
	
	return FullPathName;
	
#endif
} // GetFullPathName


//***************************************************************************
//
// Name:    GetFileName
//
// Purpose:  Extract a file name from a path.
//
// Example:  CString FileName = fs.GetFileName("c:\\foo\\bar\\what.txt");
//          ASSERT(FileName == "what.txt");
//
// Notes:    None
//
//***************************************************************************
CString
CFileSystem::GetFileName(const CString& PathAndFileName)
{
	CString FileName = PathAndFileName;  // Copy to make modifications.
	
	// Find the last "\" in the string and return everything after it.
	int nIndex = FileName.Find('\\');
	while (nIndex != -1)
	{
		FileName = FileName.Right(FileName.GetLength() - nIndex - 1);
		
		nIndex = FileName.Find('\\');
	} // while
	
	return FileName;
} // GetFileName


//***************************************************************************
//
// Name:    GetPath
//
// Purpose:  To return the directory from a path.
//
// Example:  CString Path = fs.GetPath("c:\\foo\\bar\\what.txt");
//          ASSERT(Path == "c:\\foo\\bar\\");
//
// Notes:    None
//
//***************************************************************************
CString
CFileSystem::GetPath(const CString& PathAndFileName, const BOOL bIncludeFileSystem /* = TRUE */)
{
	CString FileName = PathAndFileName;
	// Copy to modify;
	CString  Path = "";
	
	// Find the last "\" in the string and return everything up to and including it.
	int nIndex = FileName.Find('\\');
	while (nIndex != -1)
	{
		Path = Path + FileName.Left(nIndex + 1);
		FileName = FileName.Right(FileName.GetLength() - nIndex - 1);
		
		nIndex = FileName.Find('\\');
	} // while
	
	
	if (!bIncludeFileSystem)
	{
		// Remove the filesystem name from the front of the path.
		int nIndex = Path.Find(':');
		if (nIndex != -1)
		{
			Path = Path.Right(Path.GetLength() - nIndex - 1);
		} // if
	} // if
	
	return Path;
} // GetPath


//***************************************************************************
//
// Name:    GetExtension
//
// Purpose:  To return the extension from a path.
//
// Example:  CString Path = fs.GetExtension("c:\\foo\\bar\\what.txt");
//          ASSERT(Path == "txt");
//          CString Path = fs.GetExtension("c:\\foo\\bar\\what.txt.john");
//          ASSERT(Path == "john");
//
// Notes:    None
//
//***************************************************************************
CString
CFileSystem::GetExtension(const CString& PathAndFileName)
{
	// Find the last "." in the string and return everything after it.
	int nIndex = PathAndFileName.ReverseFind('.');
	CString strExt;
	
	if (nIndex > 0 && nIndex < (PathAndFileName.GetLength() - 1))
		strExt = PathAndFileName.Right(PathAndFileName.GetLength() - nIndex - 1);
	else
		strExt.Empty();
	
	return strExt;
} // GetExtension


//***************************************************************************
//
// Name:    GetFileSystem
//
// Purpose:  To return the filesystem name from the given path.
//
// Example:  CString Path = fs.GetFileSystem("c:\\foo\\bar\\what.txt");
//          ASSERT(Path == "c:\\");
//          CString Path = fs.GetFileSystem("c:");
//          ASSERT(Path == "c:");
//          CString Path = fs.GetFileSystem("\\foo\\bar\\what.txt");
//          ASSERT(Path == "");
//
// Notes:    None
//
//***************************************************************************
CString
CFileSystem::GetFileSystem(const CString& PathAndFileName)
{
	CString FileSystem;
	
	// Find the last "." in the string and return everything after it.
	int nIndex = PathAndFileName.Find(_T(":\\"));
	if (nIndex != -1)
	{
		FileSystem = PathAndFileName.Left(nIndex + 2);
	} // if
	else
	{
		nIndex = PathAndFileName.Find(':');
		if (nIndex != -1)
		{
			FileSystem = PathAndFileName.Left(nIndex + 1);
		} // if
	} // else
	
	return FileSystem;
} // GetFileSystem


//***************************************************************************
//
// Name:    GetBaseFileName
//
// Purpose:  To return the filename (without extension) given a path.
//
// Example:  CString Path = fs.GetBaseFileName("c:\\foo\\bar\\what.txt");
//          ASSERT(Path == "what");
//          CString Path = fs.GetBaseFileName("what.txt");
//          ASSERT(Path == "what");
//
// Notes:    None
//
//***************************************************************************
CString
CFileSystem::GetBaseFileName(const CString& PathAndFileName)
{
	CString FileName = GetFileName(PathAndFileName);
	CString Ext = GetExtension(FileName);
	
	if (FileName.Find('.') != -1)
		return FileName.Left(FileName.GetLength() - Ext.GetLength() - 1);
	else
		return FileName;
} // GetBaseFileName


//***************************************************************************
//
// Name:    FileExists
//
// Purpose:  To determine if a file exists.
//
// Example:
//          BOOL bExists = fs.FileExists("c:\\foo\\bar\\what.txt");
//
// Notes:    A directory is considered a file for purposes of existance.
//
//***************************************************************************
BOOL
CFileSystem::FileExists(const CString& PathAndFileName)
{
	CFileStatus FileStatus;
	
	if (CFile::GetStatus(PathAndFileName, FileStatus) == TRUE)
	{
		return TRUE;
	} // if
	else
	{
		return FALSE;
	} // if
} // FileExists


//***************************************************************************
//
// Name:    FileSystemExists
//
// Purpose:  To determine if a filesystem exists.
//
// Example:
//          BOOL bExists = fs.FileSystemExists("c:\\");
//
// Notes:    None.
//
//***************************************************************************
BOOL
CFileSystem::FileSystemExists(const CString& FileSystemName)
{
	CString TempFSName = FileSystemName;  // cast away the const.
	TempFSName.MakeLower();
	
	BOOL bRetVal = FALSE;
	CStringList *pFSList = GetFileSystemList();
	for (POSITION pos = pFSList->GetHeadPosition(); pos != NULL; )
	{
		CString FSName = pFSList->GetNext(pos);
		if (FSName == TempFSName)
		{
			bRetVal = TRUE;
			break;
		} // if
	} // for
	
	delete pFSList;
	pFSList = NULL;
	
	return bRetVal;
} // FileSystemExists


//***************************************************************************
//
// Name:    DirectoryExists
//
// Purpose: To determine if a directory exists.
//
// Example:
//          BOOL bExists = fs.DirectoryExists("c:\\temp");
//
// Notes:   None.
//
//***************************************************************************
BOOL
CFileSystem::DirectoryExists(const CString& Path)
{
#ifdef WIN32
	DWORD dwRetVal = GetFileAttributes(Path);
	if (dwRetVal == 0xFFFFFFFF)
		return FALSE;
	else
		return TRUE;
#else
	CFileStatus FileStatus;
	
	if (CFile::GetStatus(Path, FileStatus) == TRUE  &&
		FileStatus.m_attribute & directory)
	{
		return TRUE;
	} // if
	else if (FileSystemExists(GetFullPathName(Path)))
	{
		return TRUE;
	} // else if
	
	return FALSE;
#endif
} // DirectoryExists


//***************************************************************************
//
// Name:    GetDirectory
//
// Purpose:  To return a list of files based on a search string, etc...
//
// Example:  CStringList *pDir = fs.GetDirectory("*.txt", normal, TRUE);
//          (void) fs.GetDirectory("*.doc", normal, TRUE, pDir);
//
// Notes:    The filenames include the path.
//
//***************************************************************************
CStringList  *
CFileSystem::GetDirectory(const CString& SearchString, const unsigned long eFileAttrib, const BOOL bRecurseSubDirs /* = FALSE */, CStringList *pStringList /* = NULL */)
{
	// If they don't pass in a list, create one.
	if (pStringList == NULL)
	{
		pStringList = new CStringList();
	} // if
	
	// Read the file list.
	CStringList *pFileList = GetFileList(SearchString, eFileAttrib);
	pStringList->AddTail(pFileList);
	delete pFileList;
	pFileList = NULL;
	
	if (bRecurseSubDirs)
	{
		CString        CurDir = GetPath(SearchString);
		CStringList *  pDirList = GetSubdirList(CurDir);
		
		// Go through the directories we just got and recurse through them too.
		for (POSITION pos = pDirList->GetHeadPosition(); pos != 0; )
		{
			CString String = pDirList->GetNext(pos);
			
			// Get file name part of search path
			CString  SearchSpec = GetFileName(SearchString);
			
			// Do the recursion.
			GetDirectory(AppendWildcard(String, SearchSpec), eFileAttrib, bRecurseSubDirs, pStringList);
		} // for
		
		delete pDirList;
		pDirList = NULL;
	} // if
	
	return pStringList;
} // GetDirectory


//***************************************************************************
//
// Name:    GetSubdirList
//
// Purpose:  To return a list of subdirectories of another directory.
//
// Example:
//          CStringList *pDirs = fs.GetSubdirList("c:\\foo", FALSE);
//
// Notes:    None
//
//***************************************************************************
CStringList *
CFileSystem::GetSubdirList(const CString& SearchDir, const BOOL bPathInName /* = TRUE */)
{
	CFileStatus FileStatus;
	
	// Read the directory list
	CStringList *  pDirList = new CStringList();
	
	CString SearchPath = AppendWildcard(SearchDir, "*.*");
	CString *  pString = GetDirectoryEntry(SearchPath, directory);
	while (pString != NULL)
	{
		CString  String;
		CString  FullPath;
		CString  CurDir = GetPath(SearchPath);
		_stprintf(FullPath.GetBufferSetLength(1024), _T("%s%s"), (const TCHAR *)CurDir, (const TCHAR *)(*pString));
		FullPath.ReleaseBuffer(-1);
		if (bPathInName)
		{
			_stprintf(String.GetBufferSetLength(1024), _T("%s%s"), (const TCHAR *)CurDir, (const TCHAR *)(*pString));
			String.ReleaseBuffer(-1);
		} // if
		else
		{
			_stprintf(String.GetBufferSetLength(1024), _T("%s"), (const TCHAR *)(*pString));
			String.ReleaseBuffer(-1);
		} // else
		
		// If it's not one of the special directories.
		if (*pString != "."  &&  *pString != "..")
		{
			// Get the file type and make sure it's a directory before we add it to the list.
			CFile::GetStatus((const TCHAR *) FullPath, FileStatus);
			if (FileStatus.m_attribute & directory)
			{
				pDirList->AddTail((const TCHAR *) String);
			} // if
		} // if
		
		// Delete the string we got back from GetDirectoryEntry
		
		delete pString;
		pString = NULL;
		
		pString = GetDirectoryEntry("", directory);
	} // while
	
#ifdef WIN32
	if (m_hFind != INVALID_HANDLE_VALUE)
	{
		VERIFY(FindClose(m_hFind));
		m_hFind = INVALID_HANDLE_VALUE;
	} // if
#endif
	
	return pDirList;
} // GetSubdirList


//***************************************************************************
//
// Name:    GetFileList
//
// Purpose:  Return a list of files given a search path and attribute.
//
// Notes:    This only searches the specified directory.
//          Use GetDirectory() to recurse subdirectories.
//          The filenames include the path.
//
// Example:
//          CStringList *pList = GetFileList("c:\\foo\\bar\\*.cpp", normal);
//
//***************************************************************************
CStringList *
CFileSystem::GetFileList(const CString& SearchString, const unsigned long eFileAttrib)
{
	CStringList *pDirList = new CStringList();
	
	// Read the file list
	CString    CurDir = GetPath(SearchString);
	CString *  pString = GetDirectoryEntry(SearchString, eFileAttrib);
	CString  String;
	while (pString != NULL)
	{
		if (*pString != "."  &&  *pString != "..")
		{
			_stprintf(String.GetBufferSetLength(1024), _T("%s%s"), (const TCHAR *)CurDir, (const TCHAR *)(*pString));
			String.ReleaseBuffer(-1);
			pDirList->AddTail((const TCHAR *) String);
		} // if
		
		// Delete the string we got back from GetDirectoryEntry
		delete pString;
		pString = NULL;
		
		pString = GetDirectoryEntry("", eFileAttrib);
	} // while
	
	
#ifdef WIN32
	
	if (m_hFind != INVALID_HANDLE_VALUE)
	{
		VERIFY(FindClose(m_hFind));
		m_hFind = INVALID_HANDLE_VALUE;
	} // if
	
#endif
	
	return pDirList;
} // GetFileList


//***************************************************************************
//
// Name:    Sort
//
// Purpose:  To sort the give string list.
//
// Exmaple:
//          fs.Sort(pDirList);
//
// Notes:    None
//
//***************************************************************************
void
CFileSystem::Sort(CStringList *pStringList, BOOL bCase)
{
	int nListSize = pStringList->GetCount();
	for (int i = 0; i < nListSize - 1; ++i)
	{
		CString String1 = pStringList->GetAt(pStringList->FindIndex(i));
		for (int j = i + 1; j < nListSize; ++j)
		{
			CString String2 = pStringList->GetAt(pStringList->FindIndex(j));
			
			if (bCase)
			{
				if (String1 > String2)
				{
					// Swap.
					pStringList->SetAt(pStringList->FindIndex(i), String2);
					pStringList->SetAt(pStringList->FindIndex(j), String1);
					String1 = String2;
				}
			}
			else
			{
				if (String1.CompareNoCase(String2) == 1)
				{
					pStringList->SetAt(pStringList->FindIndex(i), String2);
					pStringList->SetAt(pStringList->FindIndex(j), String1);
					String1 = String2;
				}
			}// else
		} // for
	} // for
} // Sort


//***************************************************************************
//
// Name:    LoadListBox
//
// Purpose:  To load a list box with the given string list.
//
// Exmaple:
//          fs.LoadListBox(pListBox, pDirList);
//
// Notes:    None
//
//***************************************************************************
void
CFileSystem::LoadListBox(CListBox *pListBox, const CStringList * pStringList)
{
	CString String;
	
	for (POSITION pos = pStringList->GetHeadPosition(); pos != 0; )
	{
		String = pStringList->GetNext(pos);
		pListBox->AddString(String);
	} // for
} // LoadListBox


//***************************************************************************
//
// Name:    LoadComboBox
//
// Purpose:  To load a combo box with the given string list.
//
// Example:
//          fs.LoadListBox(pComboBox1, pDirList);
//
// Notes:    None
//
//***************************************************************************
void
CFileSystem::LoadComboBox(CComboBox *pComboBox, const CStringList * pStringList)
{
	CString String;
	
	for (POSITION pos = pStringList->GetHeadPosition(); pos != 0; )
	{
		String = pStringList->GetNext(pos);
		pComboBox->AddString(String);
	} // for
} // LoadComboBox


//***************************************************************************
//
// Name:    AppendWildcard
//
// Purpose:  To append a wildcard to a path.  It takes into account
//          whether the path has a backslash at the end.
//
// Example:
//          CString foo = fs.AppendWildcard("c:\\foo\\bar", "*.txt");
//          ASSERT(foo == "c:\\foo\\bar\\*.txt");
//          CString foo = fs.AppendWildcard("c:\\foo\\bar\\", "*.txt");
//          ASSERT(foo == "c:\\foo\\bar\\*.txt");
//
// Notes:    If the path is the empty string, the Wildcard is returned.
//
//***************************************************************************
CString
CFileSystem::AppendWildcard(const CString& Path, const CString& Wildcard)
{
	int nLength = Path.GetLength();
	CString RetVal;
	
	if (nLength == 0  ||  Path.GetAt(nLength - 1) == '\\')
	{
		RetVal = Path + Wildcard;
	} // if
	else
	{
		RetVal = Path + "\\" + Wildcard;
	} // else
	
	return RetVal;
} // AppendWildcard

//***************************************************************************
//
// Name:    Scan
//
// Purpose:  To locate a window from within a Window
//
// Exmaple:
//          fs.Scan( pWnd );
//
// Notes:    None
//
//***************************************************************************
void
CFileSystem::Scan(CWnd* pWnd)
{
	if (pWnd)
	{
		CWnd* pChild = pWnd->GetWindow(GW_CHILD);
		while (pChild)
		{
			// test here
			Scan(pChild);
			pChild = pChild->GetWindow(GW_HWNDNEXT);
		}
	}
}

HANDLE CFileSystem::FindFirstChildDir(LPCTSTR szPath, WIN32_FIND_DATA* lpFindData)
{
	BOOL bFound;
	HANDLE hFindFile = FindFirstFile(szPath, lpFindData);
	
	if (hFindFile != INVALID_HANDLE_VALUE)
	{
		bFound = IsChildDir(lpFindData);
		
		if (!bFound)
			bFound = FindNextChildDir(hFindFile, lpFindData);
		
		if (!bFound)
		{
			FindClose(hFindFile);
			hFindFile = INVALID_HANDLE_VALUE;
		}
	}
	
	return hFindFile;
}

BOOL CFileSystem::FindNextChildDir(HANDLE hFindFile, WIN32_FIND_DATA* lpFindData)
{
	BOOL bFound = FALSE;
	
	do
	{
		bFound = FindNextFile(hFindFile, lpFindData);
	}while (bFound && !IsChildDir(lpFindData));
	
	return bFound;
}

BOOL CFileSystem::IsChildDir(WIN32_FIND_DATA * lpFindData)
{
	return (
		(lpFindData->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
		(lpFindData->cFileName[0] != _TEXT('.')));
}

BOOL CFileSystem::IsDirectoryExist(const CString& Path)
{
	DWORD dwRetVal = GetFileAttributes(Path);
	if (dwRetVal == 0xFFFFFFFF)
		return FALSE;
	else
		return TRUE;
} 

// count files under given directory(including sub-directories if bRecurse is set TRUE)
void CFileSystem::CountFiles(const CString& strDir, int& nFileCount, const CString& strFileType, BOOL bRecurse)
{
	HANDLE hFind;
	WIN32_FIND_DATA FindData;
	BOOL bIsDir;
	CString strSrchPath(strDir);	// path to retrieve files from
	CString strNextDir;
	
	// add '\\' to end of string 
	if ("\\" != strSrchPath.Right(1))
		strSrchPath += "\\";
	
	strSrchPath += strFileType;
	
	hFind = FindFirstFile(strSrchPath, &FindData);
	BOOL fOK = (hFind != INVALID_HANDLE_VALUE);
	while (fOK)
	{
		bIsDir = FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
		if (!bIsDir)
		{
			++nFileCount;
		}
		fOK = FindNextFile(hFind, &FindData);
	}								
	
	if (hFind != INVALID_HANDLE_VALUE)
		FindClose(hFind);
	
	if (bRecurse)
	{
		strSrchPath = strDir;
		
		// add '\\' to end of string 
		if ("\\" != strSrchPath.Right(1))
			strSrchPath += "\\";
		
		strSrchPath += "*.*";
		
		hFind = FindFirstChildDir(strSrchPath, &FindData);
		fOK = (hFind != INVALID_HANDLE_VALUE);
		while (fOK)
		{
			strNextDir.GetBuffer(_MAX_DIR);
			strNextDir.Format("%s\\%s\\", strDir, FindData.cFileName);
			strNextDir.ReleaseBuffer();
			CountFiles(strNextDir, nFileCount, strFileType, bRecurse); 	
			fOK = FindNextChildDir(hFind, &FindData);
		}
		
		if (hFind != INVALID_HANDLE_VALUE)
			FindClose(hFind);
	}
}

CString CFileSystem::ComputerName()
{
	const int size = MAX_COMPUTERNAME_LENGTH + 5;
	TCHAR buffer[size];
	DWORD Size = size;
	VERIFY(GetComputerName(buffer, &Size));
	return CString(buffer);
}

CString CFileSystem::UserName()
{
	const int size = 100;
	TCHAR buffer[size];
	DWORD Size = size;
	VERIFY(GetUserName(buffer, &Size));
	return CString(buffer);
}

// Callback function called by SHBrowseForFolder's browse control
// after initialization and when selection changes
//
int __stdcall CDirDialog::BrowseCtrlCallback(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
    CDirDialog* pDirDialogObj = (CDirDialog*)lpData;
    if (uMsg == BFFM_INITIALIZED)
    {
        if (! pDirDialogObj->m_strSelDir.IsEmpty())
            ::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM)(LPCTSTR)(pDirDialogObj->m_strSelDir));
        if (! pDirDialogObj->m_strWindowTitle.IsEmpty())
            ::SetWindowText(hwnd, (LPCTSTR) pDirDialogObj->m_strWindowTitle);
    }
    else if (uMsg == BFFM_SELCHANGED)
    {
        LPITEMIDLIST pidl = (LPITEMIDLIST) lParam;
        char selection[MAX_PATH];
        if (! ::SHGetPathFromIDList(pidl, selection))
            selection[0] = '\0';
		
        CString csStatusText;
        BOOL bOk = pDirDialogObj->SelChanged(selection, csStatusText);
		
        if (pDirDialogObj->m_bStatus)
            ::SendMessage(hwnd, BFFM_SETSTATUSTEXT , 0, (LPARAM)(LPCSTR)csStatusText);
		
        ::SendMessage(hwnd, BFFM_ENABLEOK, 0, bOk);
    }
	return 0;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDirDialog::CDirDialog()
{
	m_bStatus = FALSE;
}

CDirDialog::~CDirDialog()
{
}

BOOL CDirDialog::DoBrowse(CWnd *pwndParent)
{
    if (! m_strSelDir.IsEmpty())
    {
        m_strSelDir.TrimRight();
        if (m_strSelDir.Right(1) == "\\" || m_strSelDir.Right(1) == "//" )
            m_strSelDir = m_strSelDir.Left(m_strSelDir.GetLength() - 1);
    }
	
    LPMALLOC pMalloc;
    if (SHGetMalloc(&pMalloc)!= NOERROR)
        return FALSE;
	
    BROWSEINFO bInfo;
    LPITEMIDLIST pidl;
    ZeroMemory((PVOID) &bInfo, sizeof(BROWSEINFO));
	
    if (!m_strInitDir.IsEmpty())
    {
        OLECHAR       olePath[MAX_PATH];
        ULONG         chEaten;
        ULONG         dwAttributes;
        HRESULT       hr;
        LPSHELLFOLDER pDesktopFolder;
        //
        // Get a pointer to the Desktop's IShellFolder interface.
        //
        if (SUCCEEDED(SHGetDesktopFolder(&pDesktopFolder)))
        {
            //
            // IShellFolder::ParseDisplayName requires the file name be in Unicode.
            //
            MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, m_strInitDir.GetBuffer(MAX_PATH), -1,
				olePath, MAX_PATH);
			
            m_strInitDir.ReleaseBuffer(-1);
            //
            // Convert the path to an ITEMIDLIST.
            //
            hr = pDesktopFolder->ParseDisplayName(NULL,
				NULL,
				olePath,
				&chEaten,
				&pidl,
				&dwAttributes);
            if (FAILED(hr))
            {
                pMalloc->Free(pidl);
                pMalloc->Release();
                return FALSE;
            }
            bInfo.pidlRoot = pidl;
        }
    }
    bInfo.hwndOwner = pwndParent == NULL ? NULL : pwndParent->GetSafeHwnd();
    bInfo.pszDisplayName = m_strPath.GetBuffer(MAX_PATH);
    bInfo.lpszTitle = (m_strTitle.IsEmpty()) ? "Open" : m_strTitle;
    bInfo.ulFlags = BIF_RETURNFSANCESTORS
		| BIF_RETURNONLYFSDIRS
		| (m_bStatus ? BIF_STATUSTEXT : 0);
	
    bInfo.lpfn = BrowseCtrlCallback;  // address of callback function
    bInfo.lParam = (LPARAM)this;      // pass address of object to callback function
	
    if ((pidl = ::SHBrowseForFolder(&bInfo)) == NULL)
    {
        return FALSE;
    }
    m_strPath.ReleaseBuffer();
    m_iImageIndex = bInfo.iImage;
	
    if (::SHGetPathFromIDList(pidl, m_strPath.GetBuffer(MAX_PATH)) == FALSE)
    {
        pMalloc->Free(pidl);
        pMalloc->Release();
        return FALSE;
    }
	
    m_strPath.ReleaseBuffer();
	
    pMalloc->Free(pidl);
    pMalloc->Release();
	
    return TRUE;
}
