// DirFileSys.h : SP99 Source File
//
// Defines behaviours for : Directory and File routines
// 
// COPYRIGHT  1989 - 1998, ICR Software, Inc.
// All rights reserved.
// Created  : 04/05/1997
// Modified : 09/03/1998 07:46:07 PM
//

#ifndef __CFILESYSTEM_H__
#define __CFILESYSTEM_H__

//
// C Extension DLL
// Initialize declaration context
//

#ifdef _CDLL
	#undef AFXAPP_DATA
	#define AFXAPP_DATA AFXAPI_DATA
	#undef AFX_DATA
	#define AFX_DATA    C_DATAEXT
#endif //_CDLL

#ifdef WIN32

#include <direct.h>
// winbase.h defines this to be (0x100000L) for some reason.
#ifdef GetFreeSpace
#undef GetFreeSpace
#endif

#else

#include "dos.h"
#include "direct.h"

#endif

#define	ARRAY_SIZE(A)(sizeof(A)/sizeof((A)[0]))

//***************************************************************************
// Definitions
//***************************************************************************
#define DRIVE_UNDETERMINED 0


class CFileSystem : public CObject
{
	// Constructors
public:
	CFileSystem();        // Constructor
	
	// Attributes
private:
	
#ifdef WIN32
	WIN32_FIND_DATA  m_FileInfo;
	HANDLE           m_hFind;
#else
	struct _find_t	m_FileInfo;
#endif
	
	int              m_nMaxFileNameLength;
	
// Implementation
public:
	virtual ~CFileSystem();        // Destructor
	
	// File/Directory Attributes
		
#ifndef WIN32
#define FILE_ATTRIBUTE_NORMAL CFile::normal
#endif
		
	enum Attribute 
	{
		normal    = FILE_ATTRIBUTE_NORMAL,  // Normal Files (Win32 API version 0x80)
			readOnly  = CFile::readOnly,        // Read-Only Files
			hidden    = CFile::hidden,          // Hidden Files
			system    = CFile::system,          // System Files
			volume    = CFile::volume,          // Volume Label
			directory  = CFile::directory,      // Subdirectory Entry
			archive    = CFile::archive,        // Archive File
			
			allfiles  = CFileSystem::normal    |    // All "files"
            CFileSystem::readOnly  |    // Used to determine "size" of directory.
            CFileSystem::hidden    |
            CFileSystem::system    |
            CFileSystem::archive,
	}; // Attribute
	
	
	// Operations
	HANDLE FindFirstChildDir(LPCTSTR szPath, WIN32_FIND_DATA* lpFindData);
	BOOL FindNextChildDir(HANDLE hFindFile, WIN32_FIND_DATA* lpFindData);
	BOOL IsChildDir(WIN32_FIND_DATA * lpFindData);
	void CountFiles(const CString& strDir, int& nFileCount, const CString& strFileType, BOOL bRecurse);
	BOOL LoadAniCursor(UINT nType);
	
	// Operations - Directory
	BOOL          IsDirectoryExist(const CString& Path);
	CString       GetCurrentDirectory(const CString& FileSystem = "");
	BOOL          ChangeDirectory(const CString& NewDirectory);
	BOOL          RenameDirectory(const CString& OldName, const CString& NewName);
	BOOL          MakeDirectory(const CString& NewDirectory);
	BOOL          MakePath(const CString& NewDirectory);
	BOOL          DeleteDirectory(const CString& Directory, const BOOL bDeleteFilesAndDirs = FALSE);
	CString		GetLastSubDirectory(const CString& PathName);
	void			VerifyPath(const CString& PathName);
	void			RecurseCopyPath(const CString& InputPathName, const CString& OutputPathName, const BOOL bRecurseSubdirs);
	CStringList * MakeSubdirList(const CString& SearchDir, const CString& OutputDir, const BOOL bPathInName);
	
	LONG          GetDirectorySize(const CString& Directory = "", const CString& WildCard = "*.*", const BOOL bRecurseSubdirs = FALSE);
	
	// Operations - FileSystem
	CString		ComputerName();
	CString		UserName();
	CString       GetCurrentFileSystem();
	BOOL          ChangeFileSystem(const CString& FileSystem);
	CStringList * GetFileSystemList();
	CString       GetVolumeLabel(const CString& FileSystem);
	LONG          GetFileSystemType(const CString& FileSystem);
	BOOL          IsReadOnlyFileSystem(const CString& FileSystem);
	LONG          GetFreeSpace(const CString& FileSystem);
	LONG          GetTotalSpace(const CString& FileSystem);
	
	// Operations - File
	CString		ViewErrorMessage(int nError);
	
	BOOL          GetFileStatus(const CString& FileName, CFileStatus& FileStatus);
	BOOL          GetFileCreateTime(const CString& FileName, CTime& time);
	BOOL          GetFileModifyTime(const CString& FileName, CTime& time);
	BOOL          GetFileAccessTime(const CString& FileName, CTime& time);
	BOOL          GetFileSize(const CString& FileName, unsigned long& lSize);
	BOOL          GetFileAttribute(const CString& FileName, BYTE& Attribute);
	
	BOOL          RenameFile(const CString& OldFileName, const CString& NewFileName);
	BOOL          DeleteFile(const CString& FileName);
	BOOL          DeleteFiles(const CString& FileSpec, const unsigned long eFileAttrib = normal);
	
	BOOL          CopyFile(const CString& SourceFileName, const CString& DestFileName, const unsigned long lBuffSize = 10240);
	BOOL          CopyFiles(const CString& FileSpec, const CString& DestPath, const BOOL bIgnoreErrors = FALSE, const unsigned long eFileAttrib = normal);
	
	// Operations - Comparison
	BOOL          CompareFiles(const CString& FileName1, const CString& FileName2, const unsigned long lBuffSize = 10240);
	BOOL          CompareDirectories(const CString& PathName1, const CString& PathName2, const BOOL bRecurseSubdirs = FALSE, const BOOL bCompareFileContents = FALSE, const unsigned long eFileAttrib = normal);
	
	// Operations - Existance
	
	BOOL          FileExists(const CString& PathAndFileName);
	BOOL          FileSystemExists(const CString& FileSystemName);
	BOOL          DirectoryExists(const CString& Path);
	
	// Operations - File/Path Name
	
	CString       GetFullPathName(const CString& PathAndFileName);
	CString       GetFileName(const CString& PathAndFileName);
	CString       GetPath(const CString& PathAndFileName, const BOOL bIncludeFileSystem = TRUE);
	CString       GetExtension(const CString& PathAndFileName);
	CString       GetFileSystem(const CString& PathAndFileName);
	CString       GetBaseFileName(const CString& PathAndFileName);
	
	CString       AppendWildcard(const CString& Path, const CString& Wildcard);
	
	// Operations - Directory Listing
	int CALLBACK BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
	BOOL BrowseForFolder(HWND hwnd, LPSTR lpBuffer, UINT uBufferSize,
		CString strDirTitle, LPCTSTR lpszInitialDir);
	
	CStringList * GetDirectory(const CString& SearchString, const unsigned long eFileAttrib, const BOOL bRecurseSubDirs = FALSE, CStringList *pStringList = NULL);
	
	CStringList * GetSubdirList(const CString& SearchDir, const BOOL bPathInName = TRUE);
	CStringList * GetFileList(const CString& SearchString, const unsigned long eFileAttrib);
	
	CString *     GetDirectoryEntry(const CString& Wildcard = "", const unsigned long eFileAttrib = normal);
	
	
	// Operations - MFC CStringList
	static void   Sort(CStringList *pStringList, BOOL bCase = TRUE);
	
	// Operations - MFC GUI
	static void   LoadListBox(CListBox *pListBox, const CStringList * pStringList);
	static void   LoadComboBox(CComboBox *pComboBox, const CStringList * pStringList);
	static void	Scan(CWnd* pWnd);
	
// Overridables
	
	
// Implementation
		
protected:
	BOOL          ChangeFileSystem(const char cFileSystem);
	BOOL          CloseFile(CFile *pFile) const;
	BOOL          GetADirectoryEntry(CString &fileName, const CString& Wildcard = "", const unsigned long eFileAttrib = normal);
}; // CFileSystem

class CDirDialog
{
public:
    CDirDialog();
    virtual ~CDirDialog();
	
    BOOL DoBrowse(CWnd *pwndParent = NULL);
	
    CString m_strWindowTitle;
    CString m_strPath;
    CString m_strInitDir;
    CString m_strSelDir;
    CString m_strTitle;
    int  m_iImageIndex;
    BOOL m_bStatus;
	
private:
    virtual BOOL SelChanged(LPCSTR lpcsSelection, CString& csStatusText) 
	{
		return TRUE;
	};
    static int __stdcall CDirDialog::BrowseCtrlCallback(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
};

//
// C Extension DLL
// Reset declaration context
//

#undef AFX_DATA
#define AFX_DATA
#undef AFXAPP_DATA
#define AFXAPP_DATA NEAR

#endif // _CFILESYSTEM_H_

