// IB_Accounts.cpp : implementation file
//

#include "stdafx.h"
#include "malloc.h"
#include "registry.h"
#include <afxsock.h>	// socket for time synch
#include "IB_Accounts.h"

#define SOCKET_CANNOT_INITIALIZE_STR " - Socket init failed"
#define SOCKET_CANNOT_CREATE_STR " - Socket not created"
#define SOCKET_CANNOT_CONNECT_STR " - Socket cannot connect"
#define SOCKET_CANNOT_RECEIVE_STR " - Socket receive problem"
#define INTERNAL_APP_ERROR "Internal application error"
#define SUCCESSFULLY_STR "Time set" 
#define CANNOT_SET_DATE_TIME_FORMAT " - Date & time cannot be set"
#define TIME_PROTOCOL 37
#define DATE_1970 2208988800

// IB_Accounts dialog

IMPLEMENT_DYNAMIC(IB_Accounts, CDialog)
IB_Accounts::IB_Accounts(CWnd* pParent /*=NULL*/)
	: CDialog(IB_Accounts::IDD, pParent)
	, m_lNumIB(1)
	, m_bThreadRunning(FALSE)
	, m_bRegistered(FALSE)
	, m_nThreadRet(-1)
	, m_nTimer(0)
	, m_strITS(_T(""))
	, m_nSessionNo(1)
	, ExpiryDate(0)
	, wMajor(0)
	, wMinor(0)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	int i;
	CRegistry reg;
	CString strRegIBDKey;

	m_strSessionNo.Format("%i",m_nSessionNo);

	// create registry keys where parameters will be saved
	i = reg.SetRootKey(HKEY_LOCAL_MACHINE);
	strRegIBDKey = "Software\\IBD" + m_strSessionNo + "\\DefaultSettings";
	i = reg.SetKey(strRegIBDKey, TRUE);

	strRegIBDKey = "Software\\IBD" + m_strSessionNo + "\\Connect";
	i = reg.SetKey(strRegIBDKey, TRUE);

	strRegIBDKey = "Software\\IBD" + m_strSessionNo + "\\LastOrders";
	i = reg.SetKey(strRegIBDKey, TRUE);
}

IB_Accounts::~IB_Accounts()
{
}

void IB_Accounts::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO1, NumIBAccs);
	DDX_Text(pDX, IDC_EDIT2, m_nSessionNo);
	DDX_Control(pDX, IDC_LIST1, m_lstIBDStatus);
}

BEGIN_MESSAGE_MAP(IB_Accounts, CDialog)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_BUTTON2, OnBnClickedButton2)
	ON_WM_TIMER()
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_CBN_SELCHANGE(IDC_COMBO1, OnCbnSelchangeCombo1)
	ON_BN_CLICKED(IDCANCEL2, OnBnClickedCancel2)
END_MESSAGE_MAP()


// IB_Accounts message handlers

void IB_Accounts::OnBnClickedOk()
{
	BOOL i;
	CRegistry Reg;
	CString strNum;
	
	UpdateData(TRUE);
	m_strSessionNo.Format("%i",m_nSessionNo);
	CString RegIBDKey = "Software\\IBD" + m_strSessionNo + "\\DefaultSettings";
	
	NumIBAccs.GetWindowText(strNum);
	m_lNumIB=atol(strNum);
	i = Reg.SetRootKey(HKEY_LOCAL_MACHINE);
	if (Reg.SetKey(RegIBDKey, FALSE)) 
	{
		i = Reg.WriteInt("NumIBAccs",m_lNumIB-1);
		i = Reg.WriteString("ITS",m_strITS);
	}

	if (m_bThreadRunning)		delete m_TSyncThread;
	KillTimer(m_nTimer);

	OnOK();
}


void IB_Accounts::OnBnClickedCancel()
{
	// if cancelled, clean up the timer and the running thread
	if (m_bThreadRunning)		delete m_TSyncThread;
	KillTimer(m_nTimer);

	m_lNumIB = 0;
	OnCancel();
}

BOOL IB_Accounts::OnInitDialog()
{
	CString str;
	CRegistry Reg;
	int i=0;
	CString RegIBDKey = "Software\\IBD" + m_strSessionNo + "\\DefaultSettings";
	Reg.SetRootKey(HKEY_LOCAL_MACHINE);
	if (Reg.SetKey(RegIBDKey, TRUE))	
	{
		m_strITS = Reg.ReadString("ITS","time.ien.it");
		i = Reg.ReadInt("NumIBAccs",0);
	}
	
	CDialog::OnInitDialog();
	NumIBAccs.SetCurSel(i);
	OnCbnSelchangeCombo1();

	SetWindowText("Number of IB Accounts to link to:");

	if (m_bRegistered == FALSE)
	{
		str.Format("Interactive Brokers API v%d.%d is installed.",wMajor,wMinor);
		m_lstIBDStatus.InsertString(0,str);
		str = "Shareware Beta version";// ending: ";
		m_lstIBDStatus.InsertString(1,str);
//		CString strExp = ExpiryDate.Format("%A, %B %d, %Y");
//		m_lstIBDStatus.InsertString(2,strExp);
	}
	else
	{
		str = "Registered";
		m_lstIBDStatus.InsertString(0,str);
	}

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	pSysMenu->AppendMenu(MF_SEPARATOR);
	if (pSysMenu != NULL)
	{
		CString strSettings;
		strSettings.LoadString(IDS_CLEAR_REG);
		if (!strSettings.IsEmpty())	pSysMenu->AppendMenu(MF_STRING, IDS_CLEAR_REG, strSettings);		
	}

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// temp 
	m_strSessionNo.Format("%i",m_nSessionNo);

//	UpdateData(FALSE);
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void IB_Accounts::OnBnClickedButton2()	// Time synch
{
	//MessageBox("Currently disabled","bug fixing in progress..."); return;
	if (m_bThreadRunning) return;	// do not click again while querying for time

	m_nThreadRet = -1;
    m_lstIBDStatus.ResetContent();
	m_bThreadRunning = TRUE;
	m_nTimer = SetTimer(1, 1000,0);	// to check thread termination every sec
	m_TSyncThread = AfxBeginThread(SyncTime, this);	// start new thread
	m_lstIBDStatus.InsertString(0,"Wait");
}

UINT IB_Accounts::SyncTime(LPVOID p)
{	// Set a pointer to function to be called in thread
	IB_Accounts *me = (IB_Accounts*)p;
	return me->SyncTime();
}

int IB_Accounts::SyncTime(void)
{
	int nRetCode = 0;

	CSocket* pSocket; // Create socket
	WORD wVersionRequested;
	WSADATA wsaData;
	FILETIME FileTime, LocalFileTime;
	SYSTEMTIME SystemTime;
	UINT iErr=0;
	DWORD sBuffer=0;
	long lnTime=0;
	__int64 Time_Int64=0;

	pSocket= new CSocket();

	if (!pSocket) return -2;

	// Initialize sockets for Berkeley Protocol
	wVersionRequested = MAKEWORD( 1, 1 );
 
	iErr = WSAStartup( wVersionRequested, &wsaData );
	if ( iErr != 0 ) 
	{	
		nRetCode = 1000;	// should not fail
	}
 
	if ( LOBYTE( wsaData.wVersion ) != 1 ||	HIBYTE( wsaData.wVersion ) != 1 ) 
	{
		WSACleanup( );
		nRetCode = 1; 
	}

	pSocket->Create(); // Create socket
	if(0!=GetLastError())
	{
		delete pSocket;
		nRetCode = 1001;
	}
		
	//pSocket->Connect(TS, TIME_PROTOCOL); // Connect to time server
	pSocket->Connect(m_strITS, TIME_PROTOCOL); // Connect to time server
	if(0!=GetLastError())	nRetCode = 1002;

	pSocket->Receive((void*)&sBuffer, sizeof(sBuffer)); // Receive date & time
	if(0!=GetLastError())
	{
		pSocket->Close(); 
		delete pSocket; 
		nRetCode = 1003;
	}		

	if (nRetCode == 0)  // 
	{
		pSocket->Close();
		delete pSocket; 
	}
	else
	{
		m_nThreadRet = nRetCode;
		m_bThreadRunning = FALSE;
		return nRetCode;
	}

	sBuffer = htonl(sBuffer); // Reverse byte ordering

	// Set system time & date		
	lnTime = (long) (sBuffer - (unsigned long) DATE_1970);
	Time_Int64 = Int32x32To64(lnTime, 10000000) + 116444736000000000;
	FileTime.dwLowDateTime = (DWORD)Time_Int64;
	FileTime.dwHighDateTime = (DWORD) (Time_Int64 >> 32);
	FileTimeToLocalFileTime(&FileTime, &LocalFileTime);
	FileTimeToSystemTime(&LocalFileTime, &SystemTime);
	BOOL bLocal = SetLocalTime( &SystemTime );

	if(0!=GetLastError())	nRetCode = 1005; // Time could not be set (maybe check user privileges)
		
	m_nThreadRet = nRetCode;
	m_bThreadRunning = FALSE;
	return m_nThreadRet;
}

void IB_Accounts::OnTimer(UINT nIDEvent)
{
	CString str;
	if (!m_bThreadRunning)
	{
		CTime today;
		switch (m_nThreadRet)
		{
			case -2:	str = "Memory allocation error!"; break;
			case -1:	str = "Thread returned nothing!"; break;
			case 1000:	str = "Can't initialize socket"; break;
			case 1001:	str = "Can't create socket"; break;
			case 1002:	str = "Socket can't connect"; break;
			case 1003:	str = "Socket can't receive"; break;
			case 1005:	str = "Can't set system time"; break;
			default:
				{
					today = CTime::GetCurrentTime();
					str = today.Format("%H:%M:%S");
				};
		}
		m_lstIBDStatus.ResetContent();
		m_lstIBDStatus.InsertString(0,str);	// display time
	}
	else
	{
		m_lstIBDStatus.GetText(0,str);
		str += ".";
		m_lstIBDStatus.ResetContent();
		m_lstIBDStatus.InsertString(0,str);
	}
	CDialog::OnTimer(nIDEvent);
}

void IB_Accounts::OnSysCommand(UINT nID, LPARAM lParam)
{
	if (nID == IDS_CLEAR_REG)
	{
		ClearRegistry();
		return;
	}
	CDialog::OnSysCommand(nID, lParam);
}

void IB_Accounts::ClearRegistry(void)
{
	int i;
	CRegistry reg;
	CString strRegIBDKey;

	reg.SetRootKey(HKEY_LOCAL_MACHINE);
	strRegIBDKey = "Software\\IBD" + m_strSessionNo + "\\DefaultSettings";
	if (reg.SetKey(strRegIBDKey, TRUE)) 
	{
		i = reg.DeleteKey(strRegIBDKey);
		i = reg.CreateKey(strRegIBDKey);
		i = reg.SetKey(strRegIBDKey,TRUE);
	}

	strRegIBDKey = "Software\\IBD" + m_strSessionNo + "\\Connect";
	if (reg.SetKey(strRegIBDKey, TRUE)) 
	{
		i = reg.DeleteKey(strRegIBDKey);
		i = reg.CreateKey(strRegIBDKey);
		i = reg.SetKey(strRegIBDKey,TRUE);
	}

	strRegIBDKey = "Software\\IBD" + m_strSessionNo + "\\LastOrders";
	if (reg.SetKey(strRegIBDKey, TRUE)) 
	{
		i = reg.DeleteKey(strRegIBDKey);
		i = reg.CreateKey(strRegIBDKey);
		i = reg.SetKey(strRegIBDKey,TRUE);
	}
}

void IB_Accounts::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
		CDialog::OnPaint();
}

void IB_Accounts::OnCbnSelchangeCombo1()
{
	CString str;
	int i = NumIBAccs.GetCurSel();
	NumIBAccs.GetLBText(i, str);

	SetWindowText("Number of IB Accounts to link to:" + str);
}

void IB_Accounts::OnBnClickedCancel2()
{
	m_bRegistered = TRUE;
}
