#ifndef ewrapper_def
#define ewrapper_def

enum TickType { BID_SIZE, BID, ASK, ASK_SIZE, LAST, LAST_SIZE, 
				HIGH, LOW, VOLUME, CLOSE };


inline bool isPrice( TickType tickType) {
	return tickType == BID || tickType == ASK || tickType == LAST;
}

typedef long TickerId;
typedef long OrderId;
struct OrderStatus;
struct Order;
struct Contract;
struct ContractDetails;
struct Execution;

class EWrapper
{
public:
	virtual void tickPrice( TickerId tickerId, TickType field, double price) = 0;
	virtual void tickSize( TickerId tickerId, TickType field, int size) = 0;
	virtual void orderStatus( OrderId orderId, const CString &status, int filled, 
        int remaining, double avgFillPrice, int permId, int parentId, double lastFillPrice) = 0;
	virtual void openOrder( OrderId orderId, const Contract &contract, const Order &order) = 0;
	virtual void winError( const CString &str, int lastError) = 0;
	virtual void connectionClosed() = 0;
	virtual void updateAccountValue(const CString& key, const CString& val, const CString& currency) = 0;
	virtual void updatePortfolio( const Contract& contract, int position, double marketPrice, double marketValue) = 0;
	virtual void updateAccountTime(const CString& timeStamp) = 0;
	virtual void nextValidId( OrderId orderId) = 0;
	virtual void contractDetails( const ContractDetails& contractDetails) = 0;
    virtual void execDetails( OrderId orderId, const Contract& contract, const Execution& execution) =0;
	virtual void error(const int id, const int errorCode, const CString errorString) = 0;
	virtual void updateMktDepth(TickerId id, int position, int operation, int side, 
				double price, int size) = 0;
	virtual void updateMktDepthL2(TickerId id, int position, CString marketMaker, int operation, 
				int side, double price, int size) = 0;

};


#endif
